@@ -43,30 +43,32 @@ is_empty_line(const char* begin, const char* end, const bool skip_empty_rows) {
   return *begin == '\n';
 }
 
-inline bool is_blank_or_comment_line(
+inline std::pair<bool, bool> is_blank_or_comment_line(
     const char* begin,
     const char* end,
     const std::string& comment,
     const bool skip_empty_rows) {
   if (!skip_empty_rows && comment.empty()) {
-    return false;
+    return std::pair<bool, bool>(false, false);
   }
 
   if (skip_empty_rows && (*begin == '\n' || *begin == '\r')) {
-    return true;
+    return std::pair<bool, bool>(true, false);
   }
 
   while (begin < end && (*begin == ' ' || *begin == '\t')) {
     ++begin;
   }
 
+  bool has_comment;
   if ((skip_empty_rows && (*begin == '\n' || *begin == '\r')) ||
-      (!comment.empty() &&
-       strncmp(begin, comment.data(), comment.size()) == 0)) {
-    return true;
+      (has_comment =
+           (!comment.empty() &&
+            strncmp(begin, comment.data(), comment.size()) == 0))) {
+    return std::pair<bool, bool>(true, has_comment);
   }
 
-  return false;
+  return std::pair<bool, bool>(false, false);
 }
 
 inline bool is_crlf(const char* buf, size_t pos, size_t end) {
@@ -134,7 +136,8 @@ static std::pair<size_t, newline_type> find_next_newline(
     const std::string& comment,
     const bool skip_empty_rows,
     bool embedded_nl,
-    const char quote) {
+    const char quote,
+    newline_type type = NA) {
 
   if (start >= source.size()) {
     return {source.size() - 1, NA};
@@ -152,15 +155,31 @@ static std::pair<size_t, newline_type> find_next_newline(
 
   const char* end = source.data() + source.size();
 
-  std::array<char, 3> query = {'\r', '\n', '\0'};
+  std::array<char, 3> query;
+  switch (type) {
+  case NA:
+    query = {'\n', '\r', '\0'};
+    break;
+  case CR:
+    query = {'\r', '\0'};
+    break;
+  case CRLF:
+  case LF:
+    query = {'\n', '\0'};
+    break;
+  }
+  bool should_skip;
   while (begin && begin < end) {
     size_t offset = strcspn(begin, query.data());
     // REprintf("%i\n", offset);
     begin += offset;
     break;
-    if (!(begin && begin + 1 < end &&
-          is_blank_or_comment_line(begin + 1, end, comment, skip_empty_rows))) {
-      break;
+    if (!(begin && begin + 1 < end)) {
+      std::tie(should_skip, std::ignore) =
+          is_blank_or_comment_line(begin + 1, end, comment, skip_empty_rows);
+      if (should_skip) {
+        break;
+      }
     }
   }
 
@@ -278,22 +297,28 @@ size_t find_first_line(
   auto begin = skip_bom(source);
   /* Skip skip parameters, comments and blank lines */
 
-  while (bool should_skip =
-             (begin < source.size() - 1 && is_blank_or_comment_line(
-                                               source.data() + begin,
-                                               source.data() + source.size(),
-                                               comment,
-                                               skip_empty_rows)) ||
-             skip > 0) {
+  bool should_skip, is_comment;
+  std::tie(should_skip, is_comment) = is_blank_or_comment_line(
+      source.data() + begin,
+      source.data() + source.size(),
+      comment,
+      skip_empty_rows);
+  while (begin < source.size() - 1 && (should_skip || skip > 0)) {
     std::tie(begin, std::ignore) = find_next_newline(
         source,
         begin,
         "",
         /* skip_empty_rows */ false,
         embedded_nl,
-        quote);
+        is_comment ? '\0' : quote); /* don't deal with quotes in comment lines*/
     ++begin;
     skip = skip > 0 ? skip - 1 : skip;
+
+    std::tie(should_skip, is_comment) = is_blank_or_comment_line(
+        source.data() + begin,
+        source.data() + source.size(),
+        comment,
+        skip_empty_rows);
   }
 
   return begin;
