@@ -70,11 +70,12 @@ public:
   }
 
   cpp11::data_frame error_table() const {
-    return cpp11::writable::data_frame({"row"_nm = rows_,
-                                        "col"_nm = columns_,
-                                        "expected"_nm = expected_,
-                                        "actual"_nm = actual_,
-                                        "file"_nm = filenames_});
+    return cpp11::writable::data_frame(
+        {"row"_nm = rows_,
+         "col"_nm = columns_,
+         "expected"_nm = expected_,
+         "actual"_nm = actual_,
+         "file"_nm = filenames_});
   }
 
   bool has_errors() const { return rows_.size() > 0; }
@@ -94,6 +95,16 @@ public:
     }
   }
 
+  void clear() {
+    std::lock_guard<std::mutex> guard(mutex_);
+    rows_.clear();
+    columns_.clear();
+    expected_.clear();
+    actual_.clear();
+    filenames_.clear();
+    parse_errors_.clear();
+  }
+
 private:
   mutable bool have_warned_ = false;
   std::mutex mutex_;
